﻿using System;
using System.ComponentModel;
using BusyIndicatorSample.Web.DomainService;
using System.Collections;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using BusyIndicatorSample.Web.DataModel;
using System.Threading;

namespace BusyIndicatorSample
{
    public class ViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        private NorthwindDomainContext context;

        BackgroundWorker bw;
        private bool entitySetIsLoading;
        private IEnumerable loadedEntitySet;
        
        public ViewModel()
        {
            this.context = new NorthwindDomainContext();
        }

        private void RefreshLoadedEntitySet()
        {
            bw = new BackgroundWorker();

            this.EntitySetIsLoading = true;
            
            bw.DoWork += (s, e) =>
            {
                // this simulates a delay in the loading proccess:
                Thread.Sleep(1000);
                
                LoadOperation load;
                if (this.SelectedEntitySet.EntityType == typeof(Customer)) load = context.Load<Customer>(context.GetCustomersQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Product)) load = context.Load<Product>(context.GetProductsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Product_Sales_for_1997)) load = context.Load<Product_Sales_for_1997>(context.GetProduct_Sales_for_1997Query());
                else if (this.SelectedEntitySet.EntityType == typeof(Employee)) load = context.Load<Employee>(context.GetEmployeesQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Alphabetical_list_of_product)) load = context.Load<Alphabetical_list_of_product>(context.GetAlphabetical_list_of_productsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Category)) load = context.Load<Category>(context.GetCategoriesQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Category_Sales_for_1997)) load = context.Load<Category_Sales_for_1997>(context.GetCategory_Sales_for_1997Query());
                else if (this.SelectedEntitySet.EntityType == typeof(CustomerDemographic)) load = context.Load<CustomerDemographic>(context.GetCustomerDemographicsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Order_Details_Extended)) load = context.Load<Order_Details_Extended>(context.GetOrder_Details_ExtendedsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Order_Detail)) load = context.Load<Order_Detail>(context.GetOrder_DetailsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Order_Subtotal)) load = context.Load<Order_Subtotal>(context.GetOrder_SubtotalsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Orders_Qry)) load = context.Load<Orders_Qry>(context.GetOrders_QriesQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Order)) load = context.Load<Order>(context.GetOrdersQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Products_Above_Average_Price)) load = context.Load<Products_Above_Average_Price>(context.GetProducts_Above_Average_PricesQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Region)) load = context.Load<Region>(context.GetRegionsQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Shipper)) load = context.Load<Shipper>(context.GetShippersQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Supplier)) load = context.Load<Supplier>(context.GetSuppliersQuery());
                else if (this.SelectedEntitySet.EntityType == typeof(Territory)) load = context.Load<Territory>(context.GetTerritoriesQuery());
                else load = null;

                if (load != null)
                {
                    load.Completed += (sa, ea) =>
                    {
                        this.LoadedEntitySet = load.AllEntities;
                        this.EntitySetIsLoading = false;
                    };
                }
                
                e.Result = load;
            };
            bw.RunWorkerCompleted += (s, e) =>
            {
                if (e.Result == null) 
                {
                    this.EntitySetIsLoading = false;
                    this.LoadedEntitySet = null;
                }               
            };
            bw.RunWorkerAsync();
        }

        private void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public IEnumerable EntitySets
        {
            get
            {
                return this.context.EntityContainer.EntitySets;
            }
        }

        private System.ServiceModel.DomainServices.Client.EntitySet entitySet;
            
        public System.ServiceModel.DomainServices.Client.EntitySet SelectedEntitySet
        {
            get
            {
                return entitySet;
            }
            set
            {
                entitySet = value;
                this.OnPropertyChanged("SelectedEntitySet");
                this.RefreshLoadedEntitySet();
            }
        }

        public IEnumerable LoadedEntitySet
        {
            get
            {
                return this.loadedEntitySet;
            }
            private set
            {
                this.loadedEntitySet = value;
                this.OnPropertyChanged("LoadedEntitySet");
            }
        }

        public bool EntitySetIsLoading
        {
            get
            {
                return this.entitySetIsLoading;
            }
            set
            {
                this.entitySetIsLoading = value;
                this.OnPropertyChanged("EntitySetIsLoading");
            }
        }
    }
}
